#------------------------------------------------------------------------------
#
#   Commandophone - Generic Controls
#
#------------------------------------------------------------------------------

from pygame import Rect, Color
from pygame import draw, font, midi #, gfxdraw as gfx
from albow.widget import Widget
from albow.controls import Control, ValueDisplay
from albow.palette_view import PaletteView

#------------------------------------------------------------------------------

class Slider(Widget, Control):

	slider_color = Color("black")
	slider_size = 5
	margin = 3
	border_width = 1

	def __init__(self, min, max, **kwds):
		Widget.__init__(self, **kwds)
		self.min_value = min
		self.max_value = max
	
	def _info(self):
		m = self.margin
		s = self.slider_size // 2
		w, h = self.size
		horz = w > h
		if horz:
			screen_range = w - 2 * m - 2 * s
		else:
			screen_range = h - 2 * m - 2 * s
		value_range = self.max_value - self.min_value
		return m, s, horz, screen_range, value_range
	
	def draw(self, surf):
		b = surf.get_rect()
		m, s, horz, screen_range, value_range = self._info()
		z = m + s + (self.value - self.min_value) * screen_range // value_range
		if horz:
			x = z
			y = b.centery
			dx = s
			dy = self.height // 2 - m
		else:
			x = b.centerx
			y = z
			dx = self.width // 2 - m
			dy = s
		color =  self.slider_color
		r = Rect(x-dx, y-dy, 2*dx, 2*dy)
		surf.fill(color, r)
		# gfxdraw anti-aliased shapes are crap.
		#points = [(x-dx, y), (x, y-dy), (x+dx, y), (x,y+dy-1)]
		##draw.polygon(surf, color, points)
		#gfx.filled_polygon(surf, points, color)
		#gfx.aapolygon(surf, points, color)
	
	def mouse_down(self, e):
		m, s, horz, screen_range, value_range = self._info()
		p = e.local
		if horz:
			z = p[0]
		else:
			z = p[1]
		v = (z - m - s) * value_range // screen_range
		self.value = max(self.min_value, min(v, self.max_value))
	
	def mouse_drag(self, e):
		self.mouse_down(e)

#------------------------------------------------------------------------------

class SpinArrows(Control, Widget):

	margin = 2

	min = 0
	max = 9
	wrap = False
	
	def __init__(self, **kwds):
		Widget.__init__(self, rect = Rect(0, 0, 10, 20))
		self.set(**kwds)
	
	def mouse_down(self, e):
		if e.local[1] < self.height // 2:
			d = 1
		else:
			d = -1
		self.change_value(d)
	
	def change_value(self, delta):
		new_value = self.value + delta
		vmin = self.min
		vmax = self.max
		if self.wrap:
			new_value = vmin + (new_value - vmin) % (vmax - vmin + 1)
		else:
			new_value = max(vmin, min(new_value, vmax))
		self.value = new_value

	def draw(self, surf):
		r = surf.get_rect()
		m = self.margin
		x1 = r.left + m
		x2 = r.centerx
		x3 = r.right - m
		y1 = r.top + m
		y2 = r.centery - m
		y3 = y2 + 2 * m
		y4 = r.bottom - m
		c = self.fg_color
		draw.polygon(surf, c, [(x1, y2), (x2, y1), (x3, y2)])
		draw.polygon(surf, c, [(x1, y3), (x2, y4), (x3, y3)])

#------------------------------------------------------------------------------

class IntDisplay(ValueDisplay):

	border_width = 1
	margin = 3

#------------------------------------------------------------------------------

class ListBox(Control, PaletteView):

	border_width = 1
	margin = 3
	_value = 0

	def __init__(self, labels, cell_width = 200, nrows = 10, ncols = 1,
			**kwds):
		self.labels = labels
		h = self.font.get_linesize()
		PaletteView.__init__(self, cell_size = (cell_width, h),
			nrows = nrows, ncols = ncols, scrolling = True, **kwds)
	
	def num_items(self):
		return len(self.labels)
	
	def draw_item(self, surf, i, rect):
		img = self.font.render(self.labels[i], True, self.fg_color)
		surf.blit(img, rect)
	
	def item_is_selected(self, i):
		return self.value == i
	
	def click_item(self, i, e):
		self.value = i
